/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-block-action.h - Describes operation to be done on a data block.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_BLOCK_ACTION_H
#define TNT_BLOCK_ACTION_H

#define TNT_BLOCK_ACTION(object) ((TntBlockAction *) object)

typedef enum
{
  TNT_BLOCK_ACTION_GET_BLOCK_HASH,
  TNT_BLOCK_ACTION_GET_BLOCK,
  TNT_BLOCK_ACTION_PUT_BLOCK
}
TntBlockActionType;

struct _TntBlockAction
{
  FluxObject         object;

  TntBlockActionType type;
  TntFileID          file_id;
  gint               block_id;
  guint              block_len;
  gpointer           block;
  gpointer           post_func;
  gpointer           post_func_data;

  gpointer           run_context;
};


TntBlockAction     *tnt_block_action_new           (TntBlockActionType type);
void                tnt_block_action_init          (TntBlockAction *block_action, TntBlockActionType type);
void                tnt_block_action_finalize      (FluxObject *block_action);

void                tnt_block_action_run           (TntBlockAction *block_action, TntDataDB *data_db,
                                                    FluxFileIO *file_io);
void                tnt_block_action_cancel        (TntBlockAction *block_action, TntDataDB *data_db);

TntBlockActionType  tnt_block_action_get_type      (TntBlockAction *block_action);
void                tnt_block_action_get_file_info (TntBlockAction *block_action, TntFileID *file_id);
void                tnt_block_action_set_file_info (TntBlockAction *block_action, const TntFileID *file_id);
gint                tnt_block_action_get_block_id  (TntBlockAction *block_action);
void                tnt_block_action_set_block_id  (TntBlockAction *block_action, gint block_id);
guint               tnt_block_action_get_block_len (TntBlockAction *block_action);
void                tnt_block_action_set_block_len (TntBlockAction *block_action, guint block_len);
gpointer            tnt_block_action_get_block     (TntBlockAction *block_action);
void                tnt_block_action_set_block     (TntBlockAction *block_action, gpointer block);
void                tnt_block_action_get_post_func (TntBlockAction *block_action, gpointer *func, gpointer *data);
void                tnt_block_action_set_post_func (TntBlockAction *block_action, gpointer func, gpointer data);

#endif  /* TNT_BLOCK_ACTION_H */
