/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-callback-list.h - List of callbacks.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_CALLBACK_LIST_H
#define TNT_CALLBACK_LIST_H

#define TNT_CALLBACK_LIST(object) ((TntCallbackList *) object)

typedef struct
{
  gpointer func;
  gpointer data;
}
TntCallback;

struct _TntCallbackList
{
  FluxObject  object;

  GList      *callback_list;
};

#define tnt_test_always_true(val) (TRUE)
#define tnt_test_true(val)        (val)
#define tnt_test_false(val)       (!(val))
#define tnt_test_nonzero(val)     tnt_test_true (val)
#define tnt_test_zero(val)        tnt_test_false (val)

TntCallbackList *tnt_callback_list_new                      (void);
void             tnt_callback_list_init                     (TntCallbackList *callback_list);
void             tnt_callback_list_finalize                 (FluxObject *callback_list);

void             tnt_callback_list_add_callback             (TntCallbackList *callback_list,
                                                             gpointer func, gpointer data);
void             tnt_callback_list_remove_callback          (TntCallbackList *callback_list,
                                                             gpointer func, gpointer data);

/* This was originally just one pair of vararg macros that did the job, but I had to
 * change it to a variable number of fixed arg macros, since there isn't enough C99
 * support to go around. Now it's just ugly. Pity, really. */

#define tnt_callback_list_run_while_1(cb_list, test_func, arg1)     \
G_STMT_START                                                        \
{                                                                   \
  GList    *__l;                                                    \
  gpointer  __result;                                               \
                                                                    \
  for (__l = cb_list->callback_list; __l; __l = g_list_next (__l))  \
  {                                                                 \
    TntCallback *__callback = __l->data;                            \
                                                                    \
    __result = ((gpointer (*)(gpointer a, ...)) __callback->func)   \
               ((gpointer) arg1, __callback->data);                 \
    if (!test_func (__result))                                      \
      break;                                                        \
  }                                                                 \
}                                                                   \
G_STMT_END

#define tnt_callback_list_run_while_2(cb_list, test_func, arg1,     \
                                      arg2)                         \
G_STMT_START                                                        \
{                                                                   \
  GList    *__l;                                                    \
  gpointer  __result;                                               \
                                                                    \
  for (__l = cb_list->callback_list; __l; __l = g_list_next (__l))  \
  {                                                                 \
    TntCallback *__callback = __l->data;                            \
                                                                    \
    __result = ((gpointer (*)(gpointer a, ...)) __callback->func)   \
               ((gpointer) arg1, arg2, __callback->data);           \
    if (!test_func (__result))                                      \
      break;                                                        \
  }                                                                 \
}                                                                   \
G_STMT_END

#define tnt_callback_list_run_while_3(cb_list, test_func, arg1,     \
                                      arg2, arg3)                   \
G_STMT_START                                                        \
{                                                                   \
  GList    *__l;                                                    \
  gpointer  __result;                                               \
                                                                    \
  for (__l = cb_list->callback_list; __l; __l = g_list_next (__l))  \
  {                                                                 \
    TntCallback *__callback = __l->data;                            \
                                                                    \
    __result = ((gpointer (*)(gpointer a, ...)) __callback->func)   \
               ((gpointer) arg1, arg2, arg3, __callback->data);     \
    if (!test_func (__result))                                      \
      break;                                                        \
  }                                                                 \
}                                                                   \
G_STMT_END

#define tnt_callback_list_run_while_4(cb_list, test_func, arg1,     \
                                      arg2, arg3, arg4)             \
G_STMT_START                                                        \
{                                                                   \
  GList    *__l;                                                    \
  gpointer  __result;                                               \
                                                                    \
  for (__l = cb_list->callback_list; __l; __l = g_list_next (__l))  \
  {                                                                 \
    TntCallback *__callback = __l->data;                            \
                                                                    \
    __result = ((gpointer (*)(gpointer a, ...)) __callback->func)   \
               ((gpointer) arg1, arg2, arg3, arg4,                  \
                __callback->data);                                  \
    if (!test_func (__result))                                      \
      break;                                                        \
  }                                                                 \
}                                                                   \
G_STMT_END

#define tnt_callback_list_run_while_with_result_1(cb_list, ret_type, ret_var,   \
                                                  ret_default, test_func, arg1) \
G_STMT_START                                                                    \
{                                                                               \
  GList    *__l;                                                                \
  ret_type  __result = ret_default;                                             \
                                                                                \
  for (__l = cb_list->callback_list; __l; __l = g_list_next (__l))              \
  {                                                                             \
    TntCallback *__callback = __l->data;                                        \
                                                                                \
    __result = ((ret_type (*)(gpointer a, ...)) __callback->func)               \
               ((gpointer) arg1, __callback->data);                             \
    if (!test_func (__result))                                                  \
      break;                                                                    \
  }                                                                             \
                                                                                \
  ret_var = __result;                                                           \
}                                                                               \
G_STMT_END

#define tnt_callback_list_run_while_with_result_2(cb_list, ret_type, ret_var,   \
                                                  ret_default, test_func, arg1, \
                                                  arg2)                         \
G_STMT_START                                                                    \
{                                                                               \
  GList    *__l;                                                                \
  ret_type  __result = ret_default;                                             \
                                                                                \
  for (__l = cb_list->callback_list; __l; __l = g_list_next (__l))              \
  {                                                                             \
    TntCallback *__callback = __l->data;                                        \
                                                                                \
    __result = ((ret_type (*)(gpointer a, ...)) __callback->func)               \
               ((gpointer) arg1, arg2, __callback->data);                       \
    if (!test_func (__result))                                                  \
      break;                                                                    \
  }                                                                             \
                                                                                \
  ret_var = __result;                                                           \
}                                                                               \
G_STMT_END

#define tnt_callback_list_run_while_with_result_3(cb_list, ret_type, ret_var,   \
                                                  ret_default, test_func, arg1, \
                                                  arg2, arg3)                   \
G_STMT_START                                                                    \
{                                                                               \
  GList    *__l;                                                                \
  ret_type  __result = ret_default;                                             \
                                                                                \
  for (__l = cb_list->callback_list; __l; __l = g_list_next (__l))              \
  {                                                                             \
    TntCallback *__callback = __l->data;                                        \
                                                                                \
    __result = ((ret_type (*)(gpointer a, ...)) __callback->func)               \
               ((gpointer) arg1, arg2, arg3, __callback->data);                 \
    if (!test_func (__result))                                                  \
      break;                                                                    \
  }                                                                             \
                                                                                \
  ret_var = __result;                                                           \
}                                                                               \
G_STMT_END

#define tnt_callback_list_run_while_with_result_4(cb_list, ret_type, ret_var,   \
                                                  ret_default, test_func, arg1, \
                                                  arg2, arg3, arg4)             \
G_STMT_START                                                                    \
{                                                                               \
  GList    *__l;                                                                \
  ret_type  __result = ret_default;                                             \
                                                                                \
  for (__l = cb_list->callback_list; __l; __l = g_list_next (__l))              \
  {                                                                             \
    TntCallback *__callback = __l->data;                                        \
                                                                                \
    __result = ((ret_type (*)(gpointer a, ...)) __callback->func)               \
               ((gpointer) arg1, arg2, arg3, arg4, __callback->data);           \
    if (!test_func (__result))                                                  \
      break;                                                                    \
  }                                                                             \
                                                                                \
  ret_var = __result;                                                           \
}                                                                               \
G_STMT_END

#endif  /* TNT_CALLBACK_LIST_H */
