/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-category.c - Official categories supported by Tentacles metadata.
 *
 * Copyright (C) 2002-2003 Joakim Ziegler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA. 
 *
 * Authors: Joakim Ziegler      <joakim@avmaria.com>
 *          Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include "tnt.h"

typedef struct
{
  char  *name;
  char **secondary_categories;
}
PrimaryCategory;

typedef struct
{
  char            *name;
  PrimaryCategory *primary_categories;
}
MediaDef;

static char *text_erotica_categories [] =
{
  "Fetish",
  "Gay",
  "Group",
  "Lesbian",
  NULL
};

static char *text_fiction_categories [] =
{
  "Novel",
  "Screenplay",
  "Short Story",
  NULL
};

static char *text_journalism_categories [] =
{
  "Feature Story",
  "News",
  "Review",
  "Report",
  NULL
};

static char *text_nonfiction_categories [] =
{
  "Historical",
  NULL
};

static char *text_technical_categories [] =
{
  "Instructional",
  "Reference",
  "Standard",
  "Whitepaper",
  NULL
};

static char *picture_adult_categories [] =
{
  "Amateur",
  "Cartoon",
  "Fetish",
  "Gay",
  "Group",
  "Lesbian",
  "Softcore",
  NULL
};

static char *picture_art_categories [] =
{
  "Drawing",
  "Cartoon",
  "Painting",
  "Sculpture",
  "Photography",
  NULL
};

static char *picture_photo_categories [] =
{
  "Abstract",
  "Aircraft",
  "Architecture",
  "City",
  "Event",
  "Interior",
  "Landscape",
  "Nature",
  "Object",
  "People",
  "Portrait",
  "Vehicle",
  NULL
};

static char *picture_comic_categories [] =
{
  "Action",
  "Adult",
  "Adventure",
  "Alternative",
  "Classic",
  "Comedy",
  "Crime",
  "Drama",
  "Fantasy",
  "Hentai",
  "Historical",
  "Horror",
  "Manga",
  "Romance",
  "Science Fiction",
  "Superhero",
  "Underground",
  "War",
  NULL
};

static char *picture_diagram_categories [] =
{
  "Architectural",
  "Electronic",
  "Database",
  "Flowchart",
  "Mechanical",
  "Network",
  NULL
};

static char *picture_promotional_categories [] =
{
  "Advertisement",
  "Political",
  "Product",
  NULL
};

static char *audio_alternative_categories [] =
{
  "Alternative Pop",
  "Alternative Rock",
  "Brit Pop",
  "Electronic Body Music",
  "Goth Rock",
  "Grunge",
  "Hardcore Punk",
  "Indie Rock",
  "Industrial",
  "Industrial Dance",
  "Lo-Fi",
  "Neo-Psychedelica",
  "New Wave",
  "Post-Grunge",
  "Post-Punk",
  "Post-Rock / Experimental",
  "Punk",
  "Punk Revival",
  "Punk-Pop",
  "Riot Grrrl",
  "Ska",
  "Space Rock",
  "Synth Pop",
  NULL
};

static char *audio_blues_categories [] =
{
  "Acoustic",
  "Blues Gospel",
  "British",
  "Chicago",
  "Classic Female",
  "Country",
  "Delta",
  "Detroit",
  "Electric",
  "Folk",
  "Harmonica",
  "Lousiana",
  "Memphis",
  "Modern Acoustic",
  "Modern Delta",
  "Modern Electric",
  "New Orleans",
  "New York",
  "Piano",
  "Slide Guitar",
  "Soul-Blues",
  "St. Louis",
  "Swamp",
  "Texas",
  "Urban",
  "Vaudeville",
  "West Coast",
  NULL
};

static char *audio_country_categories [] =
{
  "Alternative",
  "Americana",
  "Appalachian Folk",
  "Bakersfield Sound",
  "Close Harmony",
  "Contemporary",
  "Boogie",
  "Gospel",
  "Folk",
  "Pop",
  "Cowboy",
  "Honky Tonk",
  "Instrumental",
  "Nashville Sound",
  "Outlaw",
  "Progressive",
  "Rodeo",
  "Square Dance",
  "String Band",
  "Traditional",
  "Truck Driving",
  "Urban Cowboy",
  "Western Swing",
  "Yodeling",
  NULL
};

static char *audio_classical_categories [] =
{
  "Ballet",
  "Baroque",
  "Chamber",
  "Choral",
  "Classicism",
  "Contemporary",
  "Concerto",
  "Gregorian Chant",
  "Keyboard",
  "Medieval",
  "Modern",
  "Opera",
  "Orchestral",
  "Post-Romanticism",
  "Renaissance",
  "Symphony",
  "Vocal",
  NULL
};

static char *audio_electronic_categories [] =
{
  "Acid House",
  "Acid Jazz",
  "Acid Techno",
  "Ambient Breakbeat",
  "Ambient Dub",
  "Ambient House",
  "Ambient Techno",
  "Bhangra Beat",
  "Club",
  "Dance",
  "Dance-Pop",
  "Dark Ambient",
  "Disco",
  "Drum and Bass",
  "Electro",
  "Euro Dance",
  "Experimental Electro",
  "Experimental Techno",
  "Freestyle",
  "Funky Breaks",
  "Garage Techno",
  "Hardcore Techno",
  "House",
  "Progressive House",
  "Rave",
  "Speed Garage",
  "Techno",
  "Trance",
  "Tribal House",
  "Trip Hop",
  NULL
};

static char *audio_ethnic_categories [] =
{
  "African",
  "Arabic",
  "Asian",
  "European Folk",
  "Gaelic",
  "Indian",
  "Latin",
  "North American Folk",
  "Reggae",
  "Slavic",
  NULL
};

static char *audio_metal_categories [] =
{
  "Alternative Metal",
  "Black Metal",
  "British Metal",
  "Death Metal",
  "Funk Metal",
  "Grindcore",
  "Hair Metal",
  "Hard Rock",
  "Heavy Metal",
  "Industrial Metal",
  "Nu Metal",
  "Progressive Metal",
  "Rap Metal",
  "Ska Metal",
  "Speed Metal",
  "Thrash",
  NULL
};

static char *audio_hiphop_categories [] =
{
  "Alternative Rap",
  "Bass Music",
  "British Rap",
  "Contemporary R&B",
  "Crossover Rap",
  "Dirty Rap",
  "East Coast Rap",
  "Electric Funk",
  "French Rap",
  "G-Funk",
  "Gangsta Rap",
  "Hardcore Rap",
  "Hip Hop",
  "Jazz-Rap",
  "Latin Rap",
  "Old School",
  "Party Rap",
  "Progressive Hip Hop",
  "Rap",
  "Southern Rap",
  "Underground Rap",
  "Urban",
  "West Coast Rap",
  NULL
};

static char *audio_jazz_categories [] =
{
  "African",
  "Afro-Cuban",
  "Avant-Garde",
  "Ballads",
  "Big Band",
  "Boogie-Woogie",
  "Brazilian",
  "Classic",
  "Contemporary",
  "Crossover",
  "Cuban",
  "Dixieland",
  "Folk-Jazz",
  "Free Jazz",
  "Fusion",
  "Jazz Blues",
  "Jazz-Funk",
  "Jazz-Pop",
  "Mainstream",
  "Mood Music",
  "New Orleans",
  "Orchestral",
  "Progressive",
  "Ragtime",
  "Smooth",
  "Soul-Jazz",
  "Standards",
  "Swing",
  "Trad",
  "Vocal",
  "World Fusion",
  NULL
};

static char *audio_nonmusical_categories [] =
{
  "Literature Reading",
  "Prank Calls",
  "Radio Play",
  "Skits",
  "Spoken Word",
  "Speech",
  "Standup Comedy",
  "Talk/Lecture",
  NULL
};

static char *audio_poprock_categories [] =
{
  "Acid Rock",
  "Arena Rock",
  "Asian Pop",
  "Blues Rock",
  "British Invasion",
  "Country Rock",
  "Euro Rock",
  "Europop",
  "Experimental Rock",
  "Folk Pop",
  "Folk Rock",
  "French Pop",
  "Girl Group",
  "Glam Rock",
  "Instrumental Rock",
  "Italian Pop",
  "Jazz-Rock",
  "Latin Rock",
  "Mersey Beat",
  "Noise Rock",
  "Progressive Rock",
  "Psychedelic",
  "Singer-Songwriter",
  "Soft Rock",
  "Southern Rock",
  "Spanish Rock",
  "Teen Idol",
  NULL
};

static char *audio_rnb_categories [] =
{
  "Beach",
  "Blue-Eyed Soul",
  "Country-Soul",
  "Deep Soul",
  "Doo Wop",
  "Funk",
  "Motown",
  "Northern Soul",
  "Philly Soul",
  "Pop-Soul",
  "Retro Soul",
  "Soul",
  "Southern Soul",
  NULL
};

static char *audio_roots_categories [] =
{
  "American Traditional Rock",
  "British Traditional Rock",
  "Garage Rock",
  "Gospel",
  "Hot Rod",
  "Pub Rock",
  "Rock And Roll",
  "Rockabilly",
  "Roots-Rock",
  "Surf",
  "Surf Revival",
  "Tex-Mex",
  NULL
};

static char *audio_movie_categories [] =
{
  "Action",
  "Adventure",
  "Animation",
  "Art",
  "Children's",
  "Comedy",
  "Crime",
  "Cult",
  "Documentary",
  "Drama",
  "Fantasy",
  "Film Noir",
  "Horror",
  "Musical",
  "Mystery",
  "Romance",
  "Sci-Fi",
  "Thriller",
  "War",
  "Western",
  NULL
};

static char *video_adult_categories [] =
{
  "Amateur",
  "Animated",
  "Fetish",
  "Gay",
  "Group",
  "Lesbian",
  "Softcore",
  NULL
};

static char *video_computer_categories [] =
{
  "3D Rendered",
  "Game Capture",
  NULL
};

static char *video_educational_categories [] =
{
  "Crafts",
  "Language",
  "Physical Activity",
  "Technical",
  NULL
};

static char *video_home_categories [] =
{
  "Barbecue",
  "Birthday",
  "Comedy",
  "Family",
  "Holidays",
  "Vacation",
  "Wedding",
  NULL
};

static char *video_journalism_categories [] =
{
  "Investigative",
  "News",
  "Report",
  "Review",
  NULL
};

static char *video_movie_categories [] =
{
  "Action",
  "Adventure",
  "Animation",
  "Art",
  "Children's",
  "Comedy",
  "Crime",
  "Cult",
  "Documentary",
  "Drama",
  "Fantasy",
  "Film Noir",
  "Horror",
  "Musical",
  "Mystery",
  "Romance",
  "Sci-Fi",
  "Thriller",
  "War",
  "Western",
  NULL
};

static char *video_music_categories [] =
{
  "Alternative",
  "Blues",
  "Country",
  "Classical",
  "Electronic / Dance",
  "Ethnic / World",
  "Hip Hop",
  "Jazz",
  "Pop / Rock",
  "Heavy Rock / Metal",
  "R&B / Soul",
  "Roots",
  NULL
};

static char *video_promo_categories [] =
{
  "Advertisement",
  "Infomercial",
  NULL
};

static char *video_short_categories [] =
{
  "Action",
  "Adventure",
  "Animation",
  "Art",
  "Children's",
  "Comedy",
  "Crime",
  "Cult",
  "Drama",
  "Documentary",
  "Fantasy",
  "Film Noir",
  "Horror",
  "Mystery",
  "Romance",
  "Sci-Fi",
  "Thriller",
  "War",
  "Western",
  NULL
};

static char *video_tv_categories [] =
{
  "Action",
  "Adventure",
  "Animation",
  "Children's",
  "Cooking",
  "Comedy",
  "Crime",
  "Cult",
  "Drama",
  "Documentary",
  "Fantasy",
  "Horror",
  "Journalism",
  "Mystery",
  "Romance",
  "Sci-Fi",
  "Science / Technology",
  "Talk Show",
  "Thriller",
  "Travel",
  "War",
  "Western",
  NULL
};

static char *software_audio_categories [] =
{
  "Compression",
  "Playing",
  "Ripping",
  "Sequencing",
  NULL
};

static char *software_devel_categories [] =
{
  "Compiler / Interpreter",
  "Debugging",
  "IDE",
  "Language",
  NULL
};

static char *software_office_categories [] =
{
  "Database",
  "Diagrams",
  "Presentation",
  "Spreadsheet",
  "Word Processing",
  NULL
};

static char *software_game_categories [] =
{
  "First Person Action",
  "Platform",
  "Puzzle",
  "Realtime Strategy",
  "Roleplaying",
  "Simulation",
  "Tactical Action",
  "Turnbased Strategy",
  "Third Person Action",
  NULL
};

static char *software_graphics_categories [] =
{
  "Bitmap Graphics",
  "Conversion",
  "Desktop Publishing",
  "Filters / Effects",
  "Structured Graphics",
  NULL
};

static char *software_netclient_categories [] =
{
  "Chat / Instant Messaging",
  "Email",
  "File Sharing",
  "FTP",
  "Usenet News",
  "Web",
  NULL
};

static char *software_netserver_categories [] =
{
  "Chat / Instant Messaging",
  "Email",
  "Database",
  "DNS",
  "FTP",
  "Groupware",
  "Revision Control",
  "Usenet News",
  "Web",
  NULL
};

static char *software_scientific_categories [] =
{
  "Astronomy",
  "Mathematics",
  "Simulation",
  NULL
};

static char *software_system_categories [] =
{
  "Driver",
  "Library",
  "Operating System",
  "System Tool",
  NULL
};

static char *software_utility_categories [] =
{
  "Compression",
  "Conversion",
  "Crypto",
  NULL
};

static char *software_video_categories [] =
{
  "Compression",
  "Conversion",
  "Editing",
  "Effects / Titles",
  "Playing",
  NULL
};

static char *other_progdata_categories [] =
{
  "CAD",
  "Desktop Publishing",
  "Source Code",
  NULL
};

static PrimaryCategory text_categories [] =
{
  { "Erotica", text_erotica_categories },
  { "Fiction", text_fiction_categories },
  { "Journalism", text_journalism_categories },
  { "Nonfiction", text_nonfiction_categories },
  { "Technical", text_technical_categories },
  { NULL, NULL }
};

static PrimaryCategory picture_categories [] =
{
  { "Adult", picture_adult_categories },
  { "Art", picture_art_categories },
  { "Photo", picture_photo_categories },
  { "Diagram", picture_diagram_categories },
  { "Promotional", picture_promotional_categories },
  { "Comic", picture_comic_categories },
  { NULL, NULL }
};

static PrimaryCategory audio_categories [] = 
{
  { "Alternative", audio_alternative_categories },
  { "Blues", audio_blues_categories },
  { "Country", audio_country_categories },
  { "Classical", audio_classical_categories },
  { "Electronic/Dance", audio_electronic_categories },
  { "Ethnic / World", audio_ethnic_categories },
  { "Heavy Rock / Metal", audio_metal_categories },
  { "Hip Hop", audio_hiphop_categories },
  { "Jazz", audio_jazz_categories },
  { "Non-Musical", audio_nonmusical_categories },
  { "Pop / Rock", audio_poprock_categories },
  { "R&B / Soul", audio_rnb_categories },
  { "Roots", audio_roots_categories },
  { "Movie Soundtrack", audio_movie_categories },
  { NULL, NULL }
};

static PrimaryCategory video_categories [] =
{
  { "Adult", video_adult_categories },
  { "Computer Generated", video_computer_categories },
  { "Educational / Instructive", video_educational_categories },
  { "Home Video", video_home_categories },
  { "Journalism", video_journalism_categories },
  { "Movie", video_movie_categories },
  { "Music Video", video_music_categories },
  { "Promotional", video_promo_categories },
  { "Short Film", video_short_categories },
  { "TV Show", video_tv_categories },
  { NULL, NULL }
};

static PrimaryCategory software_categories [] =
{
  { "Audio", software_audio_categories },
  { "Development", software_devel_categories },
  { "Office", software_office_categories },
  { "Game", software_game_categories },
  { "Graphics", software_graphics_categories },
  { "Network Client", software_netclient_categories },
  { "Network Server", software_netserver_categories },
  { "Scientific", software_scientific_categories },
  { "System", software_system_categories },
  { "Utility", software_utility_categories },
  { "Video", software_video_categories },
  { NULL, NULL }
};

static PrimaryCategory other_categories [] =
{
  { "Program Data", other_progdata_categories },
  { NULL, NULL }
};


static MediaDef media_defs [] =
{
  { "Unknown", NULL},
  { "Text", text_categories },
  { "Picture", picture_categories },
  { "Audio", audio_categories },
  { "Video", video_categories },
  { "Software", software_categories },
  { "Other", other_categories },
  { NULL, NULL }
};

gint
tnt_category_get_n_labels (TntMediaType type, gint primary_category)
{
  PrimaryCategory  *p_cat;
  gchar           **s_cat;
  gint              n;

  if (type >= TNT_NUM_MEDIA_TYPES)
    return 0;

  p_cat = media_defs [type].primary_categories;
  if (!p_cat)
    return 1;  /* Unknown */

  for (n = 0; p_cat [n].name; n++)
    ;

  n += 1;  /* Unknown */

  if (primary_category < 0)
    return n;

  if (primary_category == 0)
    return 1;  /* Unknown */

  if (primary_category >= n)
    return 0;

  s_cat = p_cat [primary_category - 1].secondary_categories;
  if (!s_cat)
    return 0;

  for (n = 0; s_cat [n]; n++)
    ;

  return n + 1;
}

static char *unknown_str = "Unknown";

const gchar *
tnt_category_get_label (TntMediaType type, gint primary_category, gint secondary_category)
{
  if (type >= TNT_NUM_MEDIA_TYPES)
    return NULL;

  if (primary_category < 0)
    return media_defs [type].name;

  if (primary_category >= tnt_category_get_n_labels (type, -1))
    return NULL;

  if (primary_category == 0)
    return unknown_str;

  if (secondary_category < 0)
    return media_defs [type].primary_categories [primary_category - 1].name;

  if (secondary_category >= tnt_category_get_n_labels (type, primary_category))
    return NULL;

  if (secondary_category == 0)
    return unknown_str;

  return media_defs [type].primary_categories [primary_category - 1].secondary_categories [secondary_category - 1];
}
