/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-config.h - Configuration storage.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_CONFIG_H
#define TNT_CONFIG_H

#define TNT_CONFIG(object) ((TntConfig *) object)

typedef struct
{
  gchar    *core_id;
  FluxTree *config_node;
  guint     error_type;

  gchar    *pretty_message;
}
TntConfigError;

struct _TntConfig
{
  FluxObject       object;

  gint             peek_changes_ref;

  gboolean         validating;
  GList           *validate_errors;

  FluxTree        *peek_tree;       /* Points to one of the two below */
  FluxTree        *committed_tree;
  FluxTree        *working_tree;

  gchar           *path;

  TntCallbackList *listeners;
  TntCallbackList *validators;
};

TntConfig   *tnt_config_new                (void);
void         tnt_config_init               (TntConfig *config);
void         tnt_config_finalize           (FluxObject *config);

void         tnt_config_attach_listener    (TntConfig *config, TntConfigListenerFunc *func, gpointer data);
void         tnt_config_detach_listener    (TntConfig *config, TntConfigListenerFunc *func, gpointer data);

void         tnt_config_attach_validator   (TntConfig *config, TntConfigValidatorFunc *func, gpointer data);
void         tnt_config_detach_validator   (TntConfig *config, TntConfigValidatorFunc *func, gpointer data);

gboolean     tnt_config_store_file         (TntConfig *config, const gchar *path, gboolean replace_file);

GList       *tnt_config_validate           (TntConfig *config);
void         tnt_config_rollback           (TntConfig *config);
void         tnt_config_commit             (TntConfig *config);

void         tnt_config_validation_error   (TntConfig *config, const gchar *core_id, FluxTree *node,
                                            guint error_type, const gchar *pretty_format, ...);

const gchar *tnt_config_error_get_core_id  (TntConfigError *config_error);
FluxTree    *tnt_config_error_get_node     (TntConfigError *config_error);
guint        tnt_config_error_get_error    (TntConfigError *config_error);
const gchar *tnt_config_error_get_message  (TntConfigError *config_error);

void         tnt_config_free_errors        (GList *error_list);

void         tnt_config_peek_changes_begin (TntConfig *config);
void         tnt_config_peek_changes_end   (TntConfig *config);

/* Config value accessors */

gboolean     tnt_config_is_changed         (TntConfig *config, ...);

gboolean     tnt_config_is_path            (TntConfig *config, FluxTree *node, ...);

FluxTree    *tnt_config_peek_node          (TntConfig *config, ...);
FluxTree    *tnt_config_edit_node          (TntConfig *config, ...);

gchar       *tnt_config_get_str            (TntConfig *config, ...);
void         tnt_config_set_str            (TntConfig *config, const gchar *value, ...);

gboolean     tnt_config_get_boolean        (TntConfig *config, ...);
void         tnt_config_set_boolean        (TntConfig *config, gboolean value, ...);

gint         tnt_config_get_int            (TntConfig *config, ...);
void         tnt_config_set_int            (TntConfig *config, gint value, ...);

guint        tnt_config_get_uint           (TntConfig *config, ...);
void         tnt_config_set_uint           (TntConfig *config, guint value, ...);

gint64       tnt_config_get_int64          (TntConfig *config, ...);
void         tnt_config_set_int64          (TntConfig *config, gint64 value, ...);

guint64      tnt_config_get_uint64         (TntConfig *config, ...);
void         tnt_config_set_uint64         (TntConfig *config, guint64 value, ...);

/* va_list versions */

gboolean     tnt_config_is_changed_v       (TntConfig *config, va_list path);

gboolean     tnt_config_is_path_v          (TntConfig *config, FluxTree *node, va_list path);

FluxTree    *tnt_config_peek_node_v        (TntConfig *config, va_list path);
FluxTree    *tnt_config_edit_node_v        (TntConfig *config, va_list path);

gchar       *tnt_config_get_str_v          (TntConfig *config, va_list path);
void         tnt_config_set_str_v          (TntConfig *config, const gchar *value, va_list path);

gboolean     tnt_config_get_boolean_v      (TntConfig *config, va_list path);
void         tnt_config_set_boolean_v      (TntConfig *config, gboolean value, va_list path);

gint         tnt_config_get_int_v          (TntConfig *config, va_list path);
void         tnt_config_set_int_v          (TntConfig *config, gint value, va_list path);

guint        tnt_config_get_uint_v         (TntConfig *config, va_list path);
void         tnt_config_set_uint_v         (TntConfig *config, guint value, va_list path);

gint64       tnt_config_get_int64_v        (TntConfig *config, va_list path);
void         tnt_config_set_int64_v        (TntConfig *config, gint64 value, va_list path);

guint64      tnt_config_get_uint64_v       (TntConfig *config, va_list path);
void         tnt_config_set_uint64_v       (TntConfig *config, guint64 value, va_list path);

#endif  /* TNT_CONFIG_H */
