/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-data-db.h - Base class for disk databases.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_DATA_DB_H
#define TNT_DATA_DB_H

#define TNT_DATA_DB(object) ((TntDataDB *) object)

struct _TntDataDB
{
  FluxObject     object;

  TntStorage    *private_storage;
  FluxRmd160    *rmd_160;

  gboolean       (*have_file_full_func)   (TntDataDB *data_db, const TntFileID *file_id);
  gboolean       (*have_file_part_func)   (TntDataDB *data_db, const TntFileID *file_id);
  gboolean       (*have_block_func)       (TntDataDB *data_db, const TntFileID *file_id, gint block_id);
  TntFileBitmap *(*get_file_bitmap_func)  (TntDataDB *data_db, const TntFileID *file_id);
  gboolean       (*get_block_hash_func)   (TntDataDB *data_db, const TntFileID *file_id,
                                           gint block_id, TntPriority priority,
                                           TntGotBlockHashFunc *func, gpointer data);
  gboolean       (*get_block_func)        (TntDataDB *data_db, const TntFileID *file_id,
                                           gint block_id, TntPriority priority,
                                           TntGotBlockFunc *func, gpointer data);
  gboolean       (*put_block_func)        (TntDataDB *data_db, const TntFileID *file_id,
                                           gint block_id, gpointer block, guint len,
                                           TntPriority priority,
                                           TntWroteBlockFunc *func, gpointer data);
  void           (*cancel_func)           (TntDataDB *data_db, gpointer func, gpointer data);
  gboolean       (*pick_random_file_func) (TntDataDB *data_db, TntFileID *file_id);
  void           (*action_done_func)      (TntDataDB *data_db, gboolean success);
};

gboolean         tnt_data_db_init                     (TntDataDB *data_db, TntStorage *private_storage);
void             tnt_data_db_finalize                 (FluxObject *data_db);

TntStorage      *tnt_data_db_get_storage              (TntDataDB *data_db);

gboolean         tnt_data_db_have_file_full           (TntDataDB *data_db, const TntFileID *file_id);
gboolean         tnt_data_db_have_file_part           (TntDataDB *data_db, const TntFileID *file_id);
gboolean         tnt_data_db_have_block               (TntDataDB *data_db, const TntFileID *file_id,
                                                       gint block_id);

TntFileBitmap   *tnt_data_db_get_file_bitmap          (TntDataDB *data_db, const TntFileID *file_id);

gboolean         tnt_data_db_get_block_hash           (TntDataDB *data_db, const TntFileID *file_id,
                                                       gint block_id, TntPriority priority,
                                                       TntGotBlockHashFunc *func, gpointer data);
gboolean         tnt_data_db_get_block                (TntDataDB *data_db, const TntFileID *file_id,
                                                       gint block_id, TntPriority priority,
                                                       TntGotBlockFunc *func, gpointer data);
gboolean         tnt_data_db_put_block                (TntDataDB *data_db, const TntFileID *file_id,
                                                       gint block_id, gpointer block, guint len,
                                                       TntPriority priority,
                                                       TntWroteBlockFunc *func, gpointer data);

void             tnt_data_db_cancel                   (TntDataDB *data_db, gpointer func, gpointer data);

gboolean         tnt_data_db_pick_random_file         (TntDataDB *data_db, TntFileID *file_id);

#endif  /* TNT_DATA_DB_H */
