/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-edge.h - Describes a connection to another node.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_EDGE_H
#define TNT_EDGE_H

#define TNT_EDGE(object) ((TntEdge *) object)

struct _TntEdge
{
  FluxObject      object;

  TntNetworkCore *network_core;

  TntTimeStamp    creation_time;

  guint64         peer_fingerprint;
  gint            remote_listening_port;
  TntTcpIO       *control_tcp_io;
  TntTcpIO       *data_tcp_io;

  /* Seen nodes are prepended (or moved) to the list head. If the
   * list reaches its limit, nodes are freed from the tail. */
  guint           reachable_nodes_count;
  GList          *reachable_nodes_list;
  GList          *reachable_nodes_last;

  /* Indexes GList nodes by node ID */
  GHashTable     *reachable_nodes_table;

  /* Set if this node is qualified as sending/receiving data by
   * user request. */
  gint            qualification_refs;
  gdouble         score;
};

gboolean        tnt_edge_init                     (TntEdge *edge, TntNetworkCore *network_core);
TntEdge        *tnt_edge_new                      (TntNetworkCore *network_core);
void            tnt_edge_finalize                 (FluxObject *edge);

TntNetworkCore *tnt_edge_get_network_core         (TntEdge *edge);

TntTimeStamp    tnt_edge_get_creation_time        (TntEdge *edge);

TntTcpIO       *tnt_edge_get_io                   (TntEdge *edge, TntStreamType type);
void            tnt_edge_set_io                   (TntEdge *edge, TntStreamType type, TntTcpIO *io);

TntNodeIP4      tnt_edge_get_peer_ip4             (TntEdge *edge);
void            tnt_edge_get_peer_address4        (TntEdge *edge, TntNodeAddress4 *address);
guint64         tnt_edge_get_peer_fingerprint     (TntEdge *edge);
void            tnt_edge_set_peer_fingerprint     (TntEdge *edge, guint64 fingerprint);
void            tnt_edge_set_remote_port          (TntEdge *edge, gint port);

gboolean        tnt_edge_have_node                (TntEdge *edge, TntNodeID node_id);
guint           tnt_edge_get_node_timestamp       (TntEdge *edge, TntNodeID node_id);
gdouble         tnt_edge_get_node_weight          (TntEdge *edge, TntNodeID node_id);
void            tnt_edge_add_node_weight          (TntEdge *edge, TntNodeID node_id, gdouble weight);

gboolean        tnt_edge_get_qualified            (TntEdge *edge);
void            tnt_edge_qualification_ref        (TntEdge *edge);
void            tnt_edge_qualification_unref      (TntEdge *edge);

void            tnt_edge_send_message             (TntEdge *edge, TntMessage *message);

void            tnt_edge_change_score             (TntEdge *edge, gdouble change);
gdouble         tnt_edge_get_score                (TntEdge *edge);

void            tnt_edge_debug_dump_routing_table (TntEdge *edge);

#endif  /* TNT_EDGE_H */
