/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-file-bitmap.h - 
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_FILE_BITMAP_H
#define TNT_FILE_BITMAP_H

#define TNT_FILE_BITMAP(object) ((TntFileBitmap *) object)

struct _TntFileBitmap
{
  FluxObject    object;

  guchar       *bitmap;
  guint         len_bits;
};

TntFileBitmap   *tnt_file_bitmap_new                  (guint len_bits);
gboolean         tnt_file_bitmap_init                 (TntFileBitmap *file_bitmap, guint len_bits);
void             tnt_file_bitmap_finalize             (FluxObject *file_bitmap);

TntFileBitmap   *tnt_file_bitmap_dup                  (TntFileBitmap *file_bitmap);

TntFileBitmap   *tnt_file_bitmap_load_from_file       (const gchar *path, guint len_bits);
gboolean         tnt_file_bitmap_save_to_file         (TntFileBitmap *file_bitmap, const gchar *path);

gboolean         tnt_file_bitmap_get_bit              (TntFileBitmap *file_bitmap, guint bit_index);
void             tnt_file_bitmap_set_bit              (TntFileBitmap *file_bitmap, guint bit_index,
                                                       gboolean value);

guchar          *tnt_file_bitmap_get_bitmap           (TntFileBitmap *file_bitmap);
void             tnt_file_bitmap_set_bitmap           (TntFileBitmap *file_bitmap, guchar *bitmap);
guint            tnt_file_bitmap_get_len_bits         (TntFileBitmap *file_bitmap);

void             tnt_file_bitmap_and                  (TntFileBitmap *target_bitmap, TntFileBitmap *src_bitmap);
void             tnt_file_bitmap_or                   (TntFileBitmap *target_bitmap, TntFileBitmap *src_bitmap);
void             tnt_file_bitmap_nand                 (TntFileBitmap *target_bitmap, TntFileBitmap *src_bitmap);

void             tnt_file_bitmap_fill_true            (TntFileBitmap *file_bitmap);
void             tnt_file_bitmap_fill_false           (TntFileBitmap *file_bitmap);

gboolean         tnt_file_bitmap_is_complete          (TntFileBitmap *file_bitmap);
gboolean         tnt_file_bitmap_is_empty             (TntFileBitmap *file_bitmap);

guint            tnt_file_bitmap_count_set_bits       (TntFileBitmap *file_bitmap);
gint             tnt_file_bitmap_get_set_bit_n        (TntFileBitmap *file_bitmap, guint n);
guint64          tnt_file_bitmap_calc_data_size       (TntFileBitmap *file_bitmap, guint file_len);

void             tnt_file_bitmap_dump                 (TntFileBitmap *file_bitmap);

#endif  /* TNT_FILE_BITMAP_H */
