/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-file-core.h -
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_FILE_CORE_H
#define TNT_FILE_CORE_H

#define TNT_FILE_CORE(object) ((TntFileCore *) (object))

struct _TntFileCore
{
  TntCore                core;

  TntNetworkCore        *network_core;

  TntFileDB             *file_db;
  TntBlockDB            *block_db;
  TntMetaDB             *meta_db;

  TntFileAssembler      *assembler;
  TntDisseminator       *disseminator;

  guint16                next_local_query_id;
  GList                 *local_queries;
  TntRetrievalList      *retrieval_list;

  GList                 *remote_queries;
  gint                   n_remote_queries;

  guint                  repeat_queries_timeout_id;
  guint                  ask_avail_timeout_id;
  guint                  ask_block_timeout_id;
  guint                  summarize_timeout_id;

  gint                   next_retrieval_avail_ask;
  gint                   running_disseminations;

  TntContextFactory     *remote_query_context_factory;
  TntContextFactory     *dissemination_context_factory;
  TntContextFactory     *incoming_file_info_context_factory;
};

/* Config validation errors */

enum
{
  TNT_FILE_ERROR_DATA_DIR_UNSET = 100,
  TNT_FILE_ERROR_DATA_DIR_RELATIVE,
  TNT_FILE_ERROR_DATA_DIR_NONEXISTENT,
  TNT_FILE_ERROR_DATA_DIR_PERMS,

  TNT_FILE_ERROR_INCOMING_DIR_UNSET,
  TNT_FILE_ERROR_INCOMING_DIR_RELATIVE,
  TNT_FILE_ERROR_INCOMING_DIR_NONEXISTENT,
  TNT_FILE_ERROR_INCOMING_DIR_PERMS,

  TNT_FILE_ERROR_SHARE_RELATIVE,
  TNT_FILE_ERROR_SHARE_NONEXISTENT,
  TNT_FILE_ERROR_SHARE_PERMS,

  TNT_FILE_ERROR_DATA_DIR_FIRST = TNT_FILE_ERROR_DATA_DIR_UNSET,
  TNT_FILE_ERROR_DATA_DIR_LAST  = TNT_FILE_ERROR_DATA_DIR_PERMS,

  TNT_FILE_ERROR_INCOMING_DIR_FIRST = TNT_FILE_ERROR_INCOMING_DIR_UNSET,
  TNT_FILE_ERROR_INCOMING_DIR_LAST  = TNT_FILE_ERROR_INCOMING_DIR_PERMS,

  TNT_FILE_ERROR_SHARE_FIRST = TNT_FILE_ERROR_SHARE_RELATIVE,
  TNT_FILE_ERROR_SHARE_LAST  = TNT_FILE_ERROR_SHARE_PERMS
};

gboolean          tnt_file_core_init                (TntFileCore *file_core, TntConfig *config);
TntFileCore      *tnt_file_core_new                 (TntConfig *config);
void              tnt_file_core_finalize            (FluxObject *file_core);

void              tnt_file_core_add_local_query     (TntFileCore *file_core, TntLocalQuery *query);
void              tnt_file_core_remove_local_query  (TntFileCore *file_core, TntLocalQuery *query);
GList            *tnt_file_core_peek_local_queries  (TntFileCore *file_core);

TntFileDB        *tnt_file_core_peek_file_db        (TntFileCore *file_core);
TntBlockDB       *tnt_file_core_peek_block_db       (TntFileCore *file_core);
TntMetaDB        *tnt_file_core_peek_meta_db        (TntFileCore *file_core);
TntRetrievalList *tnt_file_core_peek_retrieval_list (TntFileCore *file_core);

#endif
