/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-hash-collider.h - Hashcash worker class
 *
 * Copyright (C) 2005 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_HASH_COLLIDER_H
#define TNT_HASH_COLLIDER_H

#define TNT_HASH_COLLIDER(object) ((TntHashCollider *) object)

/* Changing these will cause protocol incompatibility and void previously generated hashcash */
#define TNT_HASH_COLLISIONS_NUM 8
#define TNT_HASH_COLLISIONS_LEN 10

/* How many bits that need to collide to claim a "collision", and how many
 * bits you can be credited for at most. Changing these won't break anything, but it
 * may render the node more or less strict with respect to other nodes on the network */
#define TNT_HASH_COLLISIONS_MIN_BITS 119
#define TNT_HASH_COLLISIONS_MAX_BITS 130

#define TNT_HASH_COLLISIONS_MAX_EXTRA_BITS \
  (TNT_HASH_COLLISIONS_NUM * (TNT_HASH_COLLISIONS_MAX_BITS - TNT_HASH_COLLISIONS_MIN_BITS))

typedef struct
{
  guint8 v [TNT_HASH_COLLISIONS_NUM] [TNT_HASH_COLLISIONS_LEN];
}
TntHashCollisions;

struct _TntHashCollider
{
  FluxObject         object;

  TntStorage        *storage;

  GMutex            *mutex;
  GCond             *cond;
  GThread           *worker_thread;

  gboolean           running;
  gboolean           reset;
  gfloat             work_rate;

  guint64            target;
  guint8             target_hash [20];

  TntHashCollisions  target_collisions;
  gint               target_collisions_n_bits [TNT_HASH_COLLISIONS_NUM];

  guint8             next_attempt [TNT_HASH_COLLISIONS_LEN];
};

void             tnt_hash_collider_init              (TntHashCollider *hash_collider, TntStorage *storage);
TntHashCollider *tnt_hash_collider_new               (TntStorage *storage);
void             tnt_hash_collider_finalize          (FluxObject *hash_collider);

gfloat           tnt_hash_collider_get_work_rate     (TntHashCollider *hash_collider);
void             tnt_hash_collider_set_work_rate     (TntHashCollider *hash_collider, gfloat work_rate);

guint64          tnt_hash_collider_get_target        (TntHashCollider *hash_collider);
void             tnt_hash_collider_set_target        (TntHashCollider *hash_collider, guint64 target);

void             tnt_hash_collider_get_collisions    (TntHashCollider *hash_collider,
                                                      TntHashCollisions *collisions);

void             tnt_hash_collider_verify_collisions (TntHashCollider *hash_collider,
                                                      guint64 target, TntHashCollisions *collisions,
                                                      gint *n_minimal_collisions, gint *n_extra_bits);

#endif  /* TNT_HASH_COLLIDER_H */
