/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-jpeg.c - JPEG metadata extraction.
 *
 * Copyright (C) 2005 Joakim Ziegler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Joakim Ziegler <joakim@avmaria.com>
 */

#include "config.h"
#include "tnt.h"

#ifdef ENABLE_JPEG

#include <string.h>
#include <glib.h>

#include <stdio.h>
#include <jpeglib.h>
#include <setjmp.h>

#ifdef G_PLATFORM_WIN32
# include "win32/win32-compat.h"
#endif

typedef struct
{
  struct jpeg_error_mgr pub;
  jmp_buf setjmp_buffer;
}
ErrorContext;

static void
output_message_handler (j_common_ptr cinfo)
{
  /* This method prevents libjpeg from spewing to stderr */
}

static void
error_exit (j_common_ptr cinfo)
{
  ErrorContext *err = (ErrorContext *) cinfo->err;

  longjmp (err->setjmp_buffer, 1);
}

gboolean
tnt_jpeg_extract_file_meta (const gchar *file_name, TntFileMeta *file_meta)
{
  struct jpeg_decompress_struct  cinfo;
  ErrorContext                   jerr;
  FILE                          *infile;
  
  if ((infile = fopen (file_name, "rb")) == NULL)
    return FALSE;

  cinfo.err = jpeg_std_error (&jerr.pub);
  jerr.pub.error_exit = error_exit;
  jerr.pub.output_message = output_message_handler;

  if (setjmp (jerr.setjmp_buffer))
  {
    jpeg_destroy_decompress (&cinfo);
    fclose (infile);
    return FALSE;
  }

  jpeg_create_decompress (&cinfo);
  jpeg_stdio_src (&cinfo, infile);

  jpeg_read_header (&cinfo, TRUE);
  jpeg_start_decompress (&cinfo);

  tnt_file_meta_set_format (file_meta, TNT_FORMAT_JPEG);
  tnt_file_meta_set_width  (file_meta, cinfo.output_width);
  tnt_file_meta_set_height (file_meta, cinfo.output_height);

  jpeg_destroy_decompress (&cinfo);
  fclose (infile);
  return TRUE;
}

#else  /* !ENABLE_JPEG */

gboolean
tnt_jpeg_extract_file_meta (const gchar *file_name, TntFileMeta *file_meta)
{
  return FALSE;
}

#endif  /* ENABLE_JPEG */
