/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-match.h - Describes a match for a query.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_MATCH_H
#define TNT_MATCH_H

#define TNT_MATCH(s) ((TntMatch *) s)

/* An item will wait at least half of this interval before displaying
 * its availability/completeness */
#define TNT_MATCH_GRACE_INTERVAL  4

#define TNT_MATCH_IS_GROUP      (1 << 0)
#define TNT_MATCH_DATA_IS_LOCAL (1 << 1)
#define TNT_MATCH_OUT_OF_GRACE  (1 << 2)

#define TNT_MATCH_HIT_BUCKETS   32

#define tnt_match_is_group(match)      (match->flags & TNT_MATCH_IS_GROUP)
#define tnt_match_is_file(match)       (!(match->flags & TNT_MATCH_IS_GROUP))
#define tnt_match_out_of_grace(match)  (match->flags & TNT_MATCH_OUT_OF_GRACE)

struct _TntMatch
{
  guint          flags;
  TntFileMeta   *file_meta;
  TntTimeStamp   first_seen;
  TntMatch      *parent;

  union
  {
    struct
    {
      TntFileID  file_id;
      guint16    avail [TNT_MATCH_HIT_BUCKETS];
    }
    file;

    struct
    {
      GPtrArray *children;
      guint32    total_blocks;
      guint32    hit_blocks;
      guint32    hit_blocks_sum;
      guint      local_files;
      guint      quality_children;
      guint      dimension_ban : 1;
    }
    group;
  }
  u;
};

TntMatch          *tnt_match_new                (void);
TntMatch          *tnt_match_dup                (TntMatch *match);
void               tnt_match_free               (TntMatch *match);
void               tnt_match_reclaim_memory     (void);

void               tnt_match_convert_to_group   (TntMatch *match);
void               tnt_match_add_child          (TntMatch *parent, TntMatch *child);
void               tnt_match_unlink             (TntMatch *match);

TntMatch          *tnt_match_get_parent         (TntMatch *match);
const GPtrArray   *tnt_match_get_children       (TntMatch *match);
gint               tnt_match_get_n_children     (TntMatch *match);
gint               tnt_match_get_n_quality_children (TntMatch *match);
gint               tnt_match_get_index          (TntMatch *match);

void               tnt_match_inc_quality_children (TntMatch *match);
void               tnt_match_dec_quality_children (TntMatch *match);
gboolean           tnt_match_get_dimension_ban  (TntMatch *group);
void               tnt_match_set_dimension_ban  (TntMatch *group, gboolean dimension_ban);

TntMatch          *tnt_match_for_all            (TntMatch *match, TntMatchTraversalFunc *func, gpointer data);
gboolean           tnt_match_for_all_remove     (TntMatch *match, TntMatchTraversalFunc *func, gpointer data);

gboolean           tnt_match_add_hit            (TntMatch *match, TntNodeID node_id, gint num_blocks);

TntAvailability    tnt_match_get_availability   (TntMatch *match);
TntCompleteness    tnt_match_get_completeness   (TntMatch *match);

const TntFileID   *tnt_match_get_file_id        (TntMatch *match);
void               tnt_match_set_file_id        (TntMatch *match, const TntFileID *file_id);

TntFileMeta       *tnt_match_get_file_meta      (TntMatch *match);

gboolean           tnt_match_is_local           (TntMatch *match);
void               tnt_match_set_local          (TntMatch *match, gboolean local);

TntTimeStamp       tnt_match_determine_grace_timeout (TntTimeStamp seen_time);

#endif  /* TNT_MATCH_H */
