/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-message-util.c - Convenience functions for network messages.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "tnt.h"

TntMessage *
tnt_message_construct (TntMessageType type, gpointer payload_struct, gpointer data_block, guint data_len,
                       guint8 ttl)
{
  TntMessage *message;

  message = tnt_message_new (type);

  if (payload_struct)
  {
    tnt_message_set_struct (message, payload_struct);
  }

  if (data_block)
  {
    tnt_message_set_data_block (message, data_block, data_len);
  }

  tnt_message_set_ttl    (message, ttl);
  tnt_message_set_direct (message, FALSE);

  return message;
}

TntRoutedMessage *
tnt_message_construct_routed (TntMessageType type, gpointer payload_struct,
                              gpointer data_block, guint data_len,
                              guint8 ttl, TntNodeID source_id, TntNodeID dest_id)
{
  TntRoutedMessage *rmessage;

  rmessage = tnt_routed_message_new (type, source_id, dest_id);

  if (payload_struct)
  {
    tnt_message_set_struct (TNT_MESSAGE (rmessage), payload_struct);
  }

  if (data_block)
  {
    tnt_message_set_data_block (TNT_MESSAGE (rmessage), data_block, data_len);
  }

  tnt_message_set_ttl (TNT_MESSAGE (rmessage), ttl);

  return rmessage;
}

TntMessage *
tnt_message_construct_direct (TntMessageType type, gpointer payload_struct,
                              gpointer data_block, guint data_len)
{
  TntMessage *message;

  message = tnt_message_new (type);

  if (payload_struct)
  {
    tnt_message_set_struct (message, payload_struct);
  }

  if (data_block)
  {
    tnt_message_set_data_block (message, data_block, data_len);
  }

  tnt_message_set_direct (message, TRUE);

  return message;
}

gboolean
tnt_message_is_routed (TntMessage *message)
{
  flux_object_check (FLUX_OBJECT (message), TNT_MESSAGE_TYPE);

  if (flux_object_get_type (FLUX_OBJECT (message)) == TNT_ROUTED_MESSAGE_TYPE)
    return TRUE;

  return FALSE;
}

gboolean
tnt_message_is_for_me (TntMessage *message, TntNodeID my_node_id)
{
  if (my_node_id == TNT_NODE_ID_INVALID)
    return FALSE;

  if (tnt_message_get_direct (message) ||
      (tnt_message_is_routed (message) &&
       tnt_routed_message_get_destination (TNT_ROUTED_MESSAGE (message)) == my_node_id))
    return TRUE;

  return FALSE;
}
