/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-message.h - Network message.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_MESSAGE_H
#define TNT_MESSAGE_H

#define TNT_MESSAGE(object) ((TntMessage *) object)

#define TNT_MESSAGE_UNKNOWN 0
#define TNT_MESSAGE_ANY     65535

#define TNT_MESSAGE_TYPE_IS_UNKNOWN(type) ((type) == TNT_MESSAGE_UNKNOWN || (type) >= TNT_MESSAGES_NUM)

struct _TntMessage
{
  FluxObject      object;

  TntDestroyFunc *destroy_func;
  gpointer        destroy_func_data;

  TntMessageType  type;
  guint           ctime;
  guint8          ttl;
  guint8          is_direct      : 1;
  guint8          is_time_warped : 1;

  gpointer        payload;
  guint           payload_len;

  gpointer        data_block;
  guint           data_block_len;
};

#define tnt_message_has_payload(message)    ((message)->payload ? TRUE : FALSE)
#define tnt_message_has_data_block(message) ((message)->data_block ? TRUE : FALSE)

gint             tnt_message_register_marshal_info    (FluxMarshalledStruct *marshal_info);
gboolean         tnt_message_register_type            (TntMessageType type, const gchar *marshal_name);

void             tnt_message_init                     (TntMessage *message, TntMessageType type);
TntMessage      *tnt_message_new                      (TntMessageType type);
void             tnt_message_finalize                 (FluxObject *message);

TntMessageType   tnt_message_get_type                 (TntMessage *message);
guint            tnt_message_get_ctime                (TntMessage *message);
guint8           tnt_message_get_ttl                  (TntMessage *message);
gboolean         tnt_message_get_direct               (TntMessage *message);
void             tnt_message_get_payload              (TntMessage *message, gpointer *payload, guint *len);
gpointer         tnt_message_get_struct               (TntMessage *message);
void             tnt_message_get_data_block           (TntMessage *message, gpointer *data_block, guint *len);
gboolean         tnt_message_is_time_warped           (TntMessage *message);

void             tnt_message_set_ttl                  (TntMessage *message, guint8 ttl);
void             tnt_message_set_direct               (TntMessage *message, gboolean direct);
void             tnt_message_set_payload              (TntMessage *message, gpointer payload, guint len);
void             tnt_message_set_struct               (TntMessage *message, gpointer struct_ptr);
void             tnt_message_set_data_block           (TntMessage *message, gpointer data_block, guint len);
void             tnt_message_set_time_warped          (TntMessage *message, gboolean is_time_warped);

guint            tnt_message_get_size_estimate        (TntMessage *message);

void             tnt_message_free_struct              (TntMessage *message, gpointer struct_ptr);

void             tnt_message_set_destroy_func         (TntMessage *message, TntDestroyFunc *func, gpointer data);

/* DEBUG */
void             tnt_message_dump                     (TntMessage *message);

#endif  /* TNT_MESSAGE_H */
