/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-net-db.h - Keeps information on nodes in network.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_NET_DB_H
#define TNT_NET_DB_H

#define TNT_NET_DB(object) ((TntNetDB *) object)

struct _TntNetDB
{
  FluxObject    object;

  TntStorage   *storage;
  GList        *node_meta_list;
  GHashTable   *node_meta_table;
  GHashTable   *blocked_addresses;
  GHashTable   *unblocked_addresses;
  GList        *address_subsets;

  gint          n_nodes;
  guint         save_timeout_id;
};

gboolean               tnt_net_db_init                     (TntNetDB *net_db, TntStorage *storage);
TntNetDB              *tnt_net_db_new                      (TntStorage *storage);
void                   tnt_net_db_finalize                 (FluxObject *net_db);

TntNodeMeta           *tnt_net_db_get_node_meta            (TntNetDB *net_db, TntNodeID node_id);
void                   tnt_net_db_set_node_meta            (TntNetDB *net_db, TntNodeID node_id,
                                                            TntNodeMeta *node_meta);
void                   tnt_net_db_forget_node              (TntNetDB *net_db, TntNodeID node_id);
TntNodeID              tnt_net_db_get_random_node_id       (TntNetDB *net_db);

void                   tnt_net_db_add_address              (TntNetDB *net_db, const TntNodeAddress4 *address);
void                   tnt_net_db_add_address_named        (TntNetDB *net_db, const gchar *name, guint port);

void                   tnt_net_db_block_address            (TntNetDB *net_db, const TntNodeAddress4 *address);
void                   tnt_net_db_unblock_address          (TntNetDB *net_db, const TntNodeAddress4 *address);

gboolean               tnt_net_db_pick_address             (TntNetDB *net_db, TntNodeIP4 match_ip,
                                                            TntNodeIP4 match_mask,
                                                            TntNodeAddress4 *node_address);
GList                 *tnt_net_db_pick_addresses           (TntNetDB *net_db, TntNodeIP4 match_ip,
                                                            TntNodeIP4 match_mask, gint n_max);

void                   tnt_net_db_rate_address             (TntNetDB *net_db, const TntNodeAddress4 *address,
                                                            gboolean good);

gint                   tnt_net_db_get_n_nodes              (TntNetDB *net_db);
gint                   tnt_net_db_get_n_addresses          (TntNetDB *net_db);

#endif  /* TNT_NET_DB_H */
