/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-retrieval.h - Describes a running file retrieval.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_RETRIEVAL_H
#define TNT_RETRIEVAL_H

#define TNT_RETRIEVAL(object) ((TntRetrieval *) object)

#define TNT_RETRIEVAL_ETA_NOT_STARTED -1
#define TNT_RETRIEVAL_ETA_UNKNOWN     -2
#define TNT_RETRIEVAL_ETA_INFINITE    -3
#define TNT_RETRIEVAL_ETA_FINISHED    -4
#define TNT_RETRIEVAL_LOWEST_VALUE    TNT_RETRIEVAL_ETA_FINISHED

struct _TntRetrieval
{
  FluxObject         object;

  TntFileID          file_id;
  TntFileMeta       *file_meta;
  TntFileBitmap     *local_bitmap;
  GSList            *remote_sources;

  guint              is_active : 1;
  guint              cps_in;

  gint               eta;
  TntTimeStamp       eta_timestamp;
  GArray            *history;

  gchar             *target_path;

  TntRetrievalFunc  *update_func;
  gpointer           update_func_data;

  TntRetrievalFunc  *completed_func;
  gpointer           completed_func_data;

  TntRetrievalFunc  *status_change_func;
  gpointer           status_change_func_data;
};

gboolean         tnt_retrieval_init                     (TntRetrieval *retrieval, const TntFileID *file_id);
TntRetrieval    *tnt_retrieval_new                      (const TntFileID *file_id);
void             tnt_retrieval_finalize                 (FluxObject *retrieval);

const TntFileID *tnt_retrieval_get_file_id              (TntRetrieval *retrieval);

const gchar     *tnt_retrieval_get_path                 (TntRetrieval *retrieval);
void             tnt_retrieval_set_path                 (TntRetrieval *retrieval, const gchar *path);

TntFileMeta     *tnt_retrieval_get_file_meta            (TntRetrieval *retrieval);
void             tnt_retrieval_set_file_meta            (TntRetrieval *retrieval, TntFileMeta *file_meta);

TntFileBitmap   *tnt_retrieval_get_local_bitmap         (TntRetrieval *retrieval);
void             tnt_retrieval_set_local_bitmap         (TntRetrieval *retrieval, TntFileBitmap *file_bitmap);

void             tnt_retrieval_update_remote_bitmap     (TntRetrieval *retrieval, TntNodeID node_id,
                                                         TntFileBitmap *file_bitmap);
TntFileBitmap   *tnt_retrieval_get_remote_bitmap        (TntRetrieval *retrieval, TntNodeID node_id);

void             tnt_retrieval_queued                   (TntRetrieval *retrieval, TntNodeID node_id,
                                                         gint block_id, TntTimeStamp remote_timestamp);
void             tnt_retrieval_unqueued                 (TntRetrieval *retrieval, TntNodeID node_id,
                                                         gint block_id, TntTimeStamp remote_timestamp);
void             tnt_retrieval_got_block                (TntRetrieval *retrieval, gint block_id);

void             tnt_retrieval_set_active               (TntRetrieval *retrieval, gboolean state);
gboolean         tnt_retrieval_is_active                (TntRetrieval *retrieval);

gboolean         tnt_retrieval_is_complete              (TntRetrieval *retrieval);
guint            tnt_retrieval_get_source_count         (TntRetrieval *retrieval);
guchar           tnt_retrieval_get_block_saturation     (TntRetrieval *retrieval, gint block_id);
gdouble          tnt_retrieval_get_file_saturation      (TntRetrieval *retrieval, gboolean remaining);

TntNodeID        tnt_retrieval_get_block_asked          (TntRetrieval *retrieval, gint block_id);
TntNodeID        tnt_retrieval_get_block_queued         (TntRetrieval *retrieval, gint block_id);
TntNodeID        tnt_retrieval_get_block_unqueued       (TntRetrieval *retrieval, gint block_id);

GSList          *tnt_retrieval_get_busy_sources         (TntRetrieval *retrieval);
gboolean         tnt_retrieval_ask_block                (TntRetrieval *retrieval, GSList *busy_list,
                                                         TntNodeID *node_id, gint *block_id);

void             tnt_retrieval_set_cps_in               (TntRetrieval *retrieval, guint cps_in);
guint            tnt_retrieval_get_cps_in               (TntRetrieval *retrieval);

gint             tnt_retrieval_get_eta                  (TntRetrieval *retrieval);

void             tnt_retrieval_poll                     (TntRetrieval *retrieval);

void             tnt_retrieval_set_update_func          (TntRetrieval *retrieval, TntRetrievalFunc *func,
                                                         gpointer data);
void             tnt_retrieval_set_completed_func       (TntRetrieval *retrieval, TntRetrievalFunc *func,
                                                         gpointer data);
void             tnt_retrieval_set_status_change_func   (TntRetrieval *retrieval, TntRetrievalFunc *func,
                                                         gpointer data);

#endif  /* TNT_RETRIEVAL_H */
