/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-storage.h -
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_STORAGE_H
#define TNT_STORAGE_H

#include <glib.h>

#ifndef G_PLATFORM_WIN32
# define TNT_STORAGE_DIR_MODE  (S_IRWXU)
# define TNT_STORAGE_FILE_MODE (S_IRUSR | S_IWUSR)
#else
# define TNT_STORAGE_DIR_MODE  (S_IRWXU)
# define TNT_STORAGE_FILE_MODE (S_IREAD | S_IWRITE)
#endif

#define TNT_STORAGE(object) ((TntStorage *) object)

struct _TntStorage
{
  FluxObject  object;

  gchar      *base_path;
  gint64      limit;
  gint64      used;
};

gboolean     tnt_storage_init          (TntStorage *storage, const gchar *base_path, gboolean allow_create);
TntStorage  *tnt_storage_new           (const gchar *base_path, gboolean allow_create);
void         tnt_storage_finalize      (FluxObject *storage);

gboolean     tnt_storage_same_location (TntStorage *storage_a, TntStorage *storage_b);

const gchar *tnt_storage_get_path      (TntStorage *storage);
gboolean     tnt_storage_set_path      (TntStorage *storage, const gchar *path);

gint64       tnt_storage_get_limit     (TntStorage *storage);
void         tnt_storage_set_limit     (TntStorage *storage, gint64 limit);

gint64       tnt_storage_get_used      (TntStorage *storage);
void         tnt_storage_set_used      (TntStorage *storage, gint64 used);

gint64       tnt_storage_get_free      (TntStorage *storage);

#endif  /* TNT_STORAGE_H */
