/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-string-index.h -
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_STRING_INDEX_H
#define TNT_STRING_INDEX_H

#define TNT_STRING_INDEX(object)        ((TntStringIndex *) (object))
#define TNT_STRING_INDEX_CURSOR(object) ((TntStringIndexCursor *) (object))

typedef enum
{
  TNT_STRING_INDEX_IDLE,
  TNT_STRING_INDEX_INSERT,
  TNT_STRING_INDEX_FIND,
  TNT_STRING_INDEX_REMOVE
}
TntStringIndexState;

struct _TntStringIndexCursor
{
  FluxCursor            cursor;

  TntStringIndex       *string_index;

  gchar                *search_key;
  gchar                *search_mask;
  FluxExtPatTreeCursor *ext_pat_cursor;
};

struct _TntStringIndex
{
  FluxObject     object;

  guint          value_len;

  FluxStorage    *key_storage;
  FluxStorage    *value_storage;
  FluxExtPatTree *ext_pat_tree;

  gpointer       context;
};

TntStringIndex       *tnt_string_index_new             (TntStorage *storage, const gchar *base_name,
                                                        guint value_len);
gboolean              tnt_string_index_init            (TntStringIndex *string_index,
                                                        TntStorage *storage, const gchar *base_name,
                                                        guint value_len);
void                  tnt_string_index_finalize        (FluxObject *string_index);

gint                  tnt_string_index_insert          (TntStringIndex *string_index,
                                                        const gchar *string, gconstpointer value);
TntStringIndexCursor *tnt_string_index_find            (TntStringIndex *string_index,
                                                        const gchar *string);
gint                  tnt_string_index_remove          (TntStringIndex *string_index,
                                                        const gchar *string, gconstpointer value);
gint                  tnt_string_index_update          (TntStringIndex *string_index,
                                                        const gchar *before, const gchar *after,
                                                        gconstpointer value);

guint                 tnt_string_index_get_file_size   (TntStringIndex *string_index);
guint                 tnt_string_index_get_free_size   (TntStringIndex *string_index);

gboolean              tnt_string_index_cursor_init     (TntStringIndexCursor *cursor,
                                                        TntStringIndex *string_index, const gchar *string);
void                  tnt_string_index_cursor_finalize (FluxObject *object);
gboolean              tnt_string_index_cursor_read     (TntStringIndexCursor *cursor, gpointer value);

#endif
