/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-types.h - Global types and definitions.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_TYPES_H
#define TNT_TYPES_H

#include <flux/flux.h>

#define TNT_PATH_NAME_LEN         512
#define TNT_NODE_NAME_LEN_MAX     64
#define TNT_FILE_ID_LEN           8
#define TNT_BLOCK_HASH_LEN        5
#define TNT_DATA_BLOCK_SIZE       (256 * 1024)

#define TNT_LISTENING_PORT        5200

#define TNT_BITMAP_FILE_NAME      "bitmap"

/* --- Classes --- */

enum
{
  TNT_CONTEXT_FACTORY_ID,
  TNT_CALLBACK_LIST_ID,
  TNT_FILE_BITMAP_ID,
  TNT_STORAGE_ID,
  TNT_CONFIG_ID,
  TNT_STRING_INDEX_ID,
  TNT_STRING_INDEX_CURSOR_ID,
  TNT_BLOCK_ACTION_ID,
  TNT_QUERY_ID,
  TNT_LOCAL_QUERY_ID,
  TNT_REMOTE_QUERY_ID,
  TNT_RETRIEVAL_ID,
  TNT_RETRIEVAL_LIST_ID,
  TNT_META_DB_ID,
  TNT_DATA_DB_ID,
  TNT_FILE_DB_ID,
  TNT_BLOCK_DB_ID,
  TNT_FILE_ASSEMBLER_ID,
  TNT_TCP_LISTENER_ID,
  TNT_TCP_IO_ID,
  TNT_BANDWIDTH_GROUP_ID,
  TNT_BANDWIDTH_GROUP_LIST_ID,
  TNT_MESSAGE_ID,
  TNT_ROUTED_MESSAGE_ID,
  TNT_MESSAGE_FILTER_ID,
  TNT_NET_DB_ID,
  TNT_EDGE_ID,
  TNT_HASH_COLLIDER_ID,
  TNT_TIME_WARP_ID,
  TNT_DISSEMINATOR_ID,
  TNT_CORE_ID,
  TNT_NETWORK_CORE_ID,
  TNT_FILE_CORE_ID,

  TNT_CLASSES_NUM
};

#define TNT_CONTEXT_FACTORY_TYPE      tnt_class_types [TNT_CONTEXT_FACTORY_ID]
#define TNT_CALLBACK_LIST_TYPE        tnt_class_types [TNT_CALLBACK_LIST_ID]
#define TNT_FILE_BITMAP_TYPE          tnt_class_types [TNT_FILE_BITMAP_ID]
#define TNT_STORAGE_TYPE              tnt_class_types [TNT_STORAGE_ID]
#define TNT_CONFIG_TYPE               tnt_class_types [TNT_CONFIG_ID]
#define TNT_STRING_INDEX_TYPE         tnt_class_types [TNT_STRING_INDEX_ID]
#define TNT_STRING_INDEX_CURSOR_TYPE  tnt_class_types [TNT_STRING_INDEX_CURSOR_ID]
#define TNT_BLOCK_ACTION_TYPE         tnt_class_types [TNT_BLOCK_ACTION_ID]
#define TNT_QUERY_TYPE                tnt_class_types [TNT_QUERY_ID]
#define TNT_LOCAL_QUERY_TYPE          tnt_class_types [TNT_LOCAL_QUERY_ID]
#define TNT_REMOTE_QUERY_TYPE         tnt_class_types [TNT_REMOTE_QUERY_ID]
#define TNT_RETRIEVAL_TYPE            tnt_class_types [TNT_RETRIEVAL_ID]
#define TNT_RETRIEVAL_LIST_TYPE       tnt_class_types [TNT_RETRIEVAL_LIST_ID]
#define TNT_META_DB_TYPE              tnt_class_types [TNT_META_DB_ID]
#define TNT_DATA_DB_TYPE              tnt_class_types [TNT_DATA_DB_ID]
#define TNT_FILE_DB_TYPE              tnt_class_types [TNT_FILE_DB_ID]
#define TNT_BLOCK_DB_TYPE             tnt_class_types [TNT_BLOCK_DB_ID]
#define TNT_FILE_ASSEMBLER_TYPE       tnt_class_types [TNT_FILE_ASSEMBLER_ID]
#define TNT_TCP_LISTENER_TYPE         tnt_class_types [TNT_TCP_LISTENER_ID]
#define TNT_TCP_IO_TYPE               tnt_class_types [TNT_TCP_IO_ID]
#define TNT_BANDWIDTH_GROUP_TYPE      tnt_class_types [TNT_BANDWIDTH_GROUP_ID]
#define TNT_BANDWIDTH_GROUP_LIST_TYPE tnt_class_types [TNT_BANDWIDTH_GROUP_LIST_ID]
#define TNT_MESSAGE_TYPE              tnt_class_types [TNT_MESSAGE_ID]
#define TNT_ROUTED_MESSAGE_TYPE       tnt_class_types [TNT_ROUTED_MESSAGE_ID]
#define TNT_MESSAGE_FILTER_TYPE       tnt_class_types [TNT_MESSAGE_FILTER_ID]
#define TNT_NET_DB_TYPE               tnt_class_types [TNT_NET_DB_ID]
#define TNT_EDGE_TYPE                 tnt_class_types [TNT_EDGE_ID]
#define TNT_HASH_COLLIDER_TYPE        tnt_class_types [TNT_HASH_COLLIDER_ID]
#define TNT_TIME_WARP_TYPE            tnt_class_types [TNT_TIME_WARP_ID]
#define TNT_DISSEMINATOR_TYPE         tnt_class_types [TNT_DISSEMINATOR_ID]
#define TNT_CORE_TYPE                 tnt_class_types [TNT_CORE_ID]
#define TNT_NETWORK_CORE_TYPE         tnt_class_types [TNT_NETWORK_CORE_ID]
#define TNT_FILE_CORE_TYPE            tnt_class_types [TNT_FILE_CORE_ID]

typedef struct _TntContextFactory          TntContextFactory;
typedef struct _TntCallbackList            TntCallbackList;
typedef struct _TntFileBitmap              TntFileBitmap;
typedef struct _TntStorage                 TntStorage;
typedef struct _TntConfig                  TntConfig;
typedef struct _TntStringIndex             TntStringIndex;
typedef struct _TntStringIndexCursor       TntStringIndexCursor;
typedef struct _TntBlockAction             TntBlockAction;
typedef struct _TntQuery                   TntQuery;
typedef struct _TntLocalQuery              TntLocalQuery;
typedef struct _TntRemoteQuery             TntRemoteQuery;
typedef struct _TntRetrieval               TntRetrieval;
typedef struct _TntRetrievalList           TntRetrievalList;
typedef struct _TntMetaDB                  TntMetaDB;
typedef struct _TntDataDB                  TntDataDB;
typedef struct _TntFileDB                  TntFileDB;
typedef struct _TntBlockDB                 TntBlockDB;
typedef struct _TntFileAssembler           TntFileAssembler;
typedef struct _TntTcpListener             TntTcpListener;
typedef struct _TntTcpIO                   TntTcpIO;
typedef struct _TntBandwidthGroup          TntBandwidthGroup;
typedef struct _TntBandwidthGroupList      TntBandwidthGroupList;
typedef struct _TntMessage                 TntMessage;
typedef struct _TntRoutedMessage           TntRoutedMessage;
typedef struct _TntMessageFilter           TntMessageFilter;
typedef struct _TntNetDB                   TntNetDB;
typedef struct _TntEdge                    TntEdge;
typedef struct _TntHashCollider            TntHashCollider;
typedef struct _TntTimeWarp                TntTimeWarp;
typedef struct _TntDisseminator            TntDisseminator;
typedef struct _TntCore                    TntCore;
typedef struct _TntNetworkCore             TntNetworkCore;
typedef struct _TntFileCore                TntFileCore;

/* Non-objects */
typedef struct _TntAsyncQueue              TntAsyncQueue;
typedef struct _TntFileMeta                TntFileMeta;

/* --- Structs --- */

typedef guint32 TntTimeStamp;

typedef struct _TntMatch TntMatch;

typedef struct
{
  guchar  hash [TNT_FILE_ID_LEN];
  guint64 len;
}
TntFileID;

#define TNT_NODE_ID_INVALID   0
#define TNT_NODE_IP_INVALID   0
#define TNT_NODE_PORT_INVALID 0

typedef guint64 TntNodeID;
typedef guint32 TntNodeIP4;
typedef guint16 TntNodePort4;

typedef struct
{
  TntNodeIP4   ip;
  TntNodePort4 port;
}
TntNodeAddress4;

typedef enum
{
  TNT_TRANSFER_DIRECTION_IN,
  TNT_TRANSFER_DIRECTION_OUT,

  TNT_TRANSFER_DIRECTION_NUM
}
TntTransferDirection;

typedef enum
{
  TNT_STREAM_TYPE_CONTROL,
  TNT_STREAM_TYPE_DATA,

  TNT_STREAM_TYPE_NUM
}
TntStreamType;

typedef enum
{
  TNT_MATCH_FIELD_ANY,

  TNT_MATCH_FIELD_FILENAME,
  TNT_MATCH_FIELD_TITLE,
  TNT_MATCH_FIELD_COLLECTION,
  TNT_MATCH_FIELD_AUTHOR,
  TNT_MATCH_FIELD_PERFORMER,
  TNT_MATCH_FIELD_KEYWORDS
}
TntMatchField;

typedef enum
{
  TNT_MEDIA_UNKNOWN,

  TNT_MEDIA_TEXT,
  TNT_MEDIA_PICTURE,
  TNT_MEDIA_AUDIO,
  TNT_MEDIA_VIDEO,
  TNT_MEDIA_SOFTWARE,
  TNT_MEDIA_OTHER,

  TNT_NUM_MEDIA_TYPES
}
TntMediaType;

typedef enum
{
  TNT_FORMAT_UNKNOWN,

  /* Human-readable text */
  TNT_FORMAT_ASCII,
  TNT_FORMAT_UNICODE,
  TNT_FORMAT_XML,
  TNT_FORMAT_HTML,
  TNT_FORMAT_SGML,

  /* Machine-readable text */
  TNT_FORMAT_MS_DOC = 100,
  TNT_FORMAT_RTF,
  TNT_FORMAT_WORD_PERFECT,

  /* Human-readable program */
  TNT_FORMAT_SOURCE = 200,
  TNT_FORMAT_SCRIPT,
  TNT_FORMAT_SOURCE_C,
  TNT_FORMAT_SOURCE_CPLUSPLUS,
  TNT_FORMAT_SOURCE_JAVA,
  TNT_FORMAT_SOURCE_DOTNET,
  TNT_FORMAT_SOURCE_RUBY,
  TNT_FORMAT_SOURCE_PYTHON,
  TNT_FORMAT_SOURCE_PERL,
  TNT_FORMAT_SOURCE_UNIX_SHELL,
  TNT_FORMAT_SOURCE_LISP,

  TNT_FORMAT_LAST_TEXT,

  /* Picture */
  TNT_FORMAT_PNG = 300,
  TNT_FORMAT_JPEG,
  TNT_FORMAT_GIF,
  TNT_FORMAT_BMP,
  TNT_FORMAT_ZIP_COMIC,
  TNT_FORMAT_RAR_COMIC,
  TNT_FORMAT_TAR_COMIC,
  TNT_FORMAT_ACE_COMIC,

  TNT_FORMAT_LAST_PICTURE,

  /* Audio */
  TNT_FORMAT_AU = 400,
  TNT_FORMAT_WAV,
  TNT_FORMAT_OGG,
  TNT_FORMAT_MP3,
  TNT_FORMAT_MIDI,

  TNT_FORMAT_LAST_AUDIO,

  /* Video */
  TNT_FORMAT_MPEG = 500,
  TNT_FORMAT_MPEG_1,
  TNT_FORMAT_MPEG_2,
  TNT_FORMAT_MPEG_3,
  TNT_FORMAT_MPEG_4,
  TNT_FORMAT_DIVX,
  TNT_FORMAT_QUICKTIME,
  TNT_FORMAT_AVI,
  TNT_FORMAT_ASF,
  TNT_FORMAT_WMF,
  TNT_FORMAT_WMV,
  TNT_FORMAT_OGM,

  TNT_FORMAT_LAST_VIDEO,

  /* Executable */
  TNT_FORMAT_EXEC_LINUX = 600,
  TNT_FORMAT_EXEC_LISP,
  TNT_FORMAT_EXEC_WINDOWS,
  TNT_FORMAT_EXEC_NES_ROM,
  TNT_FORMAT_EXEC_SNES_ROM,
  TNT_FORMAT_EXEC_NES64_ROM,
  TNT_FORMAT_EXEC_NGB_ROM,
  TNT_FORMAT_EXEC_NGBA_ROM,

  /* Executable helpers (libraries, makefiles, etc) */
  TNT_FORMAT_EXEC_HELPER_LINUX = 700,
  TNT_FORMAT_GETTEXT,

  TNT_FORMAT_LAST_SOFTWARE,
  
  /* Archive */
  TNT_FORMAT_ZIP = 800,
  TNT_FORMAT_TAR,
  TNT_FORMAT_ZOO,
  TNT_FORMAT_ARJ,
  TNT_FORMAT_LHA,
  TNT_FORMAT_RAR,
  TNT_FORMAT_ARC,
  TNT_FORMAT_GZIP,
  TNT_FORMAT_BZIP,
  TNT_FORMAT_BZIP2,
  TNT_FORMAT_SHELL_ARCHIVE,
  TNT_FORMAT_UNIX_COMPRESS,
  TNT_FORMAT_MS_CAB,
  TNT_FORMAT_RPM,
  TNT_FORMAT_DEB,
  TNT_FORMAT_ISO9660,
  TNT_FORMAT_MISC_CDROM,
  TNT_FORMAT_ACE
}
TntFormatType;

typedef struct
{
  TntTimeStamp    last_seen;

  TntNodeAddress4 primary_address;
  gint            primary_rating;

  TntNodeAddress4 secondary_address;
  gint            secondary_rating;

  guint16         flags;
}
TntNodeMeta;

/* --- Simple types --- */

typedef enum
{
  TNT_PRIORITY_LOW    = -100,
  TNT_PRIORITY_NORMAL =    0,
  TNT_PRIORITY_HIGH   =  100
}
TntPriority;

typedef enum
{
  TNT_AVAILABILITY_UNKNOWN,
  TNT_AVAILABILITY_NONE,
  TNT_AVAILABILITY_VERY_LOW,
  TNT_AVAILABILITY_LOW,
  TNT_AVAILABILITY_MEDIUM,
  TNT_AVAILABILITY_HIGH,
  TNT_AVAILABILITY_VERY_HIGH
}
TntAvailability;

typedef enum
{
  TNT_COMPLETENESS_UNKNOWN,
  TNT_COMPLETENESS_NONE,
  TNT_COMPLETENESS_PARTIAL,
  TNT_COMPLETENESS_COMPLETE
}
TntCompleteness;

typedef enum
{
  TNT_LOG_SUBSYSTEM_CACHE,
  TNT_LOG_SUBSYSTEM_NETWORK,
  TNT_LOG_SUBSYSTEM_FILE,
  TNT_LOG_SUBSYSTEM_SEARCH
}
TntLogSubsystem;

typedef enum
{
  TNT_LOG_LEVEL_SPEW,
  TNT_LOG_LEVEL_DEBUG,
  TNT_LOG_LEVEL_LOW,
  TNT_LOG_LEVEL_NORMAL,
  TNT_LOG_LEVEL_HIGH
}
TntLogLevel;

typedef enum
{
  TNT_LOG_MOOD_GOOD,
  TNT_LOG_MOOD_PLEASING,
  TNT_LOG_MOOD_NEUTRAL,
  TNT_LOG_MOOD_DISAPPOINTING,
  TNT_LOG_MOOD_BAD
}
TntLogMood;

typedef guint32 TntLogFlags;

#define TNT_LOG_FOLLOWUP  (1 << 0)

#define TNT_LOG_MASK_ANY  0xffffffff
#define TNT_LOG_MASK_NONE 0x00000000

#define TNT_MESSAGE_FLAG_IS_DIRECT      (1 << 0)
#define TNT_MESSAGE_FLAG_HAS_TTL        (1 << 1)
#define TNT_MESSAGE_FLAG_HAS_SOURCE     (1 << 2)
#define TNT_MESSAGE_FLAG_HAS_DEST       (1 << 3)
#define TNT_MESSAGE_FLAG_HAS_PAYLOAD    (1 << 4)
#define TNT_MESSAGE_FLAG_HAS_DATA_BLOCK (1 << 5)
#define TNT_MESSAGE_FLAG_HAS_SIGNATURE  (1 << 6)
#define TNT_MESSAGE_FLAG_IS_ENCRYPTED   (1 << 7)

typedef guint TntMessageType;

typedef enum
{
  TNT_BANDWIDTH_ADJUSTMENT_NONE,
  TNT_BANDWIDTH_ADJUSTMENT_INCREASE,
  TNT_BANDWIDTH_ADJUSTMENT_DECREASE
}
TntBandwidthAdjustment;

typedef GNode *TntFileDBHandle;

/* --- Callback types --- */

typedef void (TntDestroyFunc)            (FluxObject *object, gpointer data);
typedef void (TntCallerFunc)             (gpointer func, gpointer data, gpointer *args);
typedef void (TntPersistentRenameFunc)   (const gchar *old_path, const gchar *new_path,
                                          gpointer data);
typedef void (TntLogFunc)                (TntLogSubsystem subsys, TntLogLevel level, TntLogMood mood,
                                          TntLogFlags flags, const gchar *message, gpointer data);
typedef void (TntTcpConnectionFunc)      (TntTcpListener *tl, TntTcpIO *tio, gpointer data);
typedef void (TntMessageInFunc)          (TntTcpIO *tcp_io, TntMessage *message, gpointer data);
typedef void (TntGotBlockHashFunc)       (TntDataDB *data_db, TntFileID *file_id, gint block_id,
                                          guint block_len, guint8 *block_hash, gpointer data);
typedef void (TntGotBlockFunc)           (TntDataDB *data_db, TntFileID *file_id, gint block_id,
                                          guint block_len, gpointer block, gpointer data);
typedef void (TntWroteBlockFunc)         (TntDataDB *data_db, TntFileID *file_id, gint block_id,
                                          guint block_len, gpointer data);
typedef gboolean (TntMatchTraversalFunc) (TntMatch *match, gpointer data);
typedef void (TntMatchFunc)              (TntLocalQuery *query, TntMatch *match, gpointer data);
typedef void (TntRetrievalFunc)          (TntRetrieval *retrieval, gpointer data);
typedef void (TntRetrievalListFunc)      (TntRetrievalList *retrieval_list, gint retrieval_n, gpointer data);
typedef void (TntStringIndexInsertFunc)  (TntStringIndex *string_index, gboolean success, gpointer data);
typedef void (TntStringIndexFindFunc)    (TntStringIndex *string_index, GArray *results, gpointer data);
typedef void (TntStringIndexRemoveFunc)  (TntStringIndex *string_index, gint n_removals, gpointer data);
typedef void (TntBandwidthGroupFunc)     (TntBandwidthGroup *bandwidth_group, gpointer data);
typedef void (TntBandwidthActionFunc)    (TntBandwidthGroup *bandwidth_group, TntTransferDirection direction,
                                          TntBandwidthAdjustment adjust, gpointer data);
typedef void (TntBandwidthGroupListFunc) (TntBandwidthGroupList *bwgl, gint bandwidth_group_n, gpointer data);
typedef void (TntHandleFunc)             (TntFileDB *file_db, TntFileDBHandle handle, gpointer data);
typedef void (TntAssemblyFunc)           (TntFileAssembler *assembler, const TntFileID *file_id,
                                          gpointer data);
typedef void (TntDispatchFunc)           (gpointer dispatch_data, gpointer callback_data);
typedef void (TntQueryDoneFunc)          (TntQuery *query, gpointer data);
typedef void (TntFileMetaFunc)           (const TntFileID *file_id, TntFileMeta *file_meta, gpointer data);
typedef void (TntFileIDFunc)             (const TntFileID *file_id, gpointer data);
typedef void (TntFileUpdateFunc)         (TntFileDB *file_db, TntFileID *file_id, gpointer data);
typedef gboolean (TntFileIDHintFunc)     (TntFileDB *file_db, const gchar *path, const gchar *name,
                                          guint64 len, TntTimeStamp timestamp, TntFileIDFunc *func,
                                          gpointer func_data, gpointer data);

typedef void (TntNewEdgeHandler)         (TntNetworkCore *network_core, TntEdge *edge, gpointer data);
typedef gboolean (TntStarvationHandler)  (TntNetworkCore *network_core, TntBandwidthGroup *group,
                                          TntTransferDirection dir, gpointer data);
typedef gboolean (TntMessageHandler)     (TntNetworkCore *network_core, TntEdge *source_edge,
                                          TntMessage *message, gpointer data);
typedef void (TntPreDataHandler)         (TntNetworkCore *network_core, TntEdge *source_edge,
                                          TntMessage *message, gpointer data);
typedef void (TntConfigListenerFunc)     (TntConfig *config, gpointer data);
typedef void (TntConfigValidatorFunc)    (TntConfig *config, gpointer data);
typedef void (TntCoreListenerFunc)       (const gchar *core_id, gpointer core, gpointer data);

#endif  /* TNT_TYPES_H */
