/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-util.h -
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_UTIL_H
#define TNT_UTIL_H

#include <sys/stat.h>

#define TNT_BITS_TO_BYTES(bits)                                            \
  (((bits) + 7) >> 3)

#define TNT_REAL_LEN_TO_BITMAP_LEN(bytes)                                  \
  ((((bytes) + TNT_DATA_BLOCK_SIZE - 1) / TNT_DATA_BLOCK_SIZE))

#define TNT_BLOCK_IN_FILE_LEN(block_id, file_len)                          \
  ((file_len) - ((block_id) * TNT_DATA_BLOCK_SIZE) < TNT_DATA_BLOCK_SIZE ? \
   (file_len) - ((block_id) * TNT_DATA_BLOCK_SIZE) : TNT_DATA_BLOCK_SIZE)

#define NIBBLE_TO_HEX_ASCII(n)                                             \
  (((n) < 0x0a) ? ((n) + '0') : ((n) - 0x0a + 'a'))

#define HEX_ASCII_TO_NIBBLE(c)                                             \
  (((c) - '0' < 10 && (c) - '0' >= 0) ? (c) - '0'      : \
   ((c) - 'a' < 6 && (c) - 'a' >= 0)  ? (c) - 'a' + 10 : \
   (-1))

#define BYTES_IN_K        1024
#define KS_IN_M           1024
#define MS_IN_G           1024
#define GS_IN_T           1024

#define BYTES_IN_M        ((guint64) BYTES_IN_K * (guint64) KS_IN_M)
#define BYTES_IN_G        ((guint64) BYTES_IN_M * (guint64) MS_IN_G)
#define BYTES_IN_T        ((guint64) BYTES_IN_G * (guint64) GS_IN_T)

#define SECONDS_IN_MINUTE 60
#define MINUTES_IN_HOUR   60
#define HOURS_IN_DAY      24
#define DAYS_IN_WEEK      7
#define DAYS_IN_MONTH     30
#define DAYS_IN_YEAR      365

#define SECONDS_IN_HOUR   (SECONDS_IN_MINUTE * MINUTES_IN_HOUR)
#define SECONDS_IN_DAY    (SECONDS_IN_HOUR * HOURS_IN_DAY)
#define SECONDS_IN_WEEK   (SECONDS_IN_DAY * DAYS_IN_WEEK)
#define SECONDS_IN_MONTH  (SECONDS_IN_DAY * DAYS_IN_MONTH)
#define SECONDS_IN_YEAR   (SECONDS_IN_DAY * DAYS_IN_YEAR)

typedef struct
{
  guint64      len;
  TntTimeStamp mtime;
  TntTimeStamp ctime;
  guint        is_file : 1;
  guint        is_dir  : 1;
}
TntStat;

gchar       *tnt_nice_path                    (const gchar *path);

gboolean     tnt_path_exists                  (const gchar *path);
gboolean     tnt_path_is_dir                  (const gchar *path);
gboolean     tnt_path_is_file                 (const gchar *file);
gboolean     tnt_dir_have_full_access         (const gchar *dir);
gboolean     tnt_dir_have_full_access_stat    (struct stat *sbuf);
gboolean     tnt_dir_have_read_access         (const gchar *dir);
gboolean     tnt_dir_have_read_access_stat    (struct stat *sbuf);
gboolean     tnt_file_have_full_access        (const gchar *file);
gboolean     tnt_file_have_full_access_stat   (struct stat *sbuf);
gboolean     tnt_file_have_read_access        (const gchar *file);
gboolean     tnt_file_have_read_access_stat   (struct stat *sbuf);

gboolean     tnt_dir_delete                   (const gchar *dir);
gboolean     tnt_dir_create                   (const gchar *dir);

gchar       *tnt_amount_print_dot_padded      (guint64 amount);
gchar       *tnt_time_describe                (guint seconds_abs);
gchar       *tnt_time_describe_delta          (gint seconds_ago);
gchar       *tnt_size_describe                (guint64 size);
gchar       *tnt_bandwidth_describe           (guint cps);

gint         tnt_strlen_limited               (const gchar *string, gint limit);
void         tnt_string_to_lower              (gchar *string);
void         tnt_string_capitalize_first      (gchar *string);
GList       *tnt_string_extract_substrings    (const gchar *string, gint min_token_len, gint max_token_len,
                                               gboolean get_min_string_end);
GList       *tnt_string_list_get_additions    (GList *list_1, GList *list_2);
gchar       *tnt_string_list_get_least_common (GList *list);
gint         tnt_levenshtein_distance         (const gchar *a, const gchar *b);

gchar       *tnt_utf8_make_matchable          (const gchar *original);
void         tnt_utf8_strlcpy                 (gchar *dest, const gchar *src, gsize dest_size);

gchar       *tnt_path_make_keyword_string     (const gchar *path);
GSList      *tnt_path_split                   (const gchar *path);
gchar       *tnt_path_join                    (GSList *pelem);
gint         tnt_compare_paths                (const gchar *path_1, const gchar *path_2);

gboolean     tnt_path_create                  (const gchar *path);
gint         tnt_open                         (const gchar *path, gboolean write,
                                               gboolean truncate, gboolean public);
void         tnt_close                        (gint fd);
gboolean     tnt_stat                         (const gchar *path, TntStat *tstat);
gboolean     tnt_lstat                        (const gchar *path, TntStat *tstat);
gboolean     tnt_fstat                        (gint fd, TntStat *sbuf);
gint64       tnt_lseek                        (gint fd, gint64 offset, gint whence);
gint         tnt_read                         (gint fd, gpointer buf, gint len);
gint         tnt_mkdir                        (const gchar *path);
gint         tnt_rename                       (const gchar *old_path, const gchar *new_path);
void         tnt_persistent_rename            (const gchar *old_path, const gchar *new_path,
                                               TntPersistentRenameFunc *rename_done_func, gpointer data);
gboolean     tnt_safe_read                    (gint fd, gpointer dest, gint len);
gboolean     tnt_safe_write                   (gint fd, gconstpointer src, gint len);

gboolean     tnt_string_is_valid_ipv4_address (const gchar *address);
gboolean     tnt_string_is_valid_address      (const gchar *address);
TntNodeIP4   tnt_string_to_ip                 (const gchar *string);
gchar       *tnt_ip_to_string                 (TntNodeIP4 ip);

TntNodeIP4   tnt_ip_addr_to_ip                (FluxIPAddr *ip_addr);
FluxIPAddr  *tnt_ip_to_ip_addr                (TntNodeIP4 ip4);

gchar       *tnt_file_id_to_string            (const TntFileID *file_id);
gboolean     tnt_string_to_file_id            (const gchar *string, TntFileID *file_id);
gchar       *tnt_block_id_to_string           (gint block_id);
gchar       *tnt_file_block_id_to_string      (const TntFileID *file_id, gint block_id);
gboolean     tnt_is_valid_file_id_string      (const gchar *string);
gboolean     tnt_is_valid_block_id_string     (const gchar *string);

TntNodeID    tnt_public_key_to_node_id        (FluxRSAPublicKey *public_key);
void         tnt_fold_hash                    (const guchar *hash, guchar *dest, guint len);
void         tnt_fold_mem                     (gconstpointer src, guint srclen, gpointer dest, guint destlen);

TntTimeStamp tnt_timestamp                    (void);

GList       *tnt_list_unique                  (GList *list);

void         tnt_slist_free_with_data         (GSList *slist);
void         tnt_list_free_with_data          (GList *list);

gchar       *tnt_filename_to_utf8_lossy       (const gchar *filename);
gchar       *tnt_filename_to_utf8_replace     (gchar *filename);

gchar       *tnt_utf8_to_filename_lossy       (gchar *string);

gpointer     tnt_alloc_block                  (gint len);
void         tnt_free_block                   (gpointer block);
gint         tnt_get_blocks_allocated         (void);

void         tnt_debug_dump_address           (const TntNodeAddress4 *address);

#endif
