/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* test-file-meta-extract.c - Test file metadata extraction
 *
 * Copyright (C) 2005 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include <glib.h>
#include <tnt.h>
#include <string.h>

static gboolean opt_verbose = FALSE;

static void
test_extract_file (const gchar *path)
{
  TntFileMeta *file_meta;

  if (opt_verbose)
  {
    g_print ("%s: ", path);
    fflush (stdout);
  }

  file_meta = tnt_file_meta_extract (path);
  if (file_meta)
  {
    if (opt_verbose)
      g_print ("Ok, format %d.\n", tnt_file_meta_get_format (file_meta));
    tnt_file_meta_free (file_meta);
  }
  else
  {
    if (opt_verbose)
      g_print ("No metadata extracted.\n");
  }
}

gint
main (gint argc, gchar *argv [])
{
  gint i;

  tnt_register_classes ();

  if (argc < 2)
  {
    g_print ("Usage: %s [-v] FILE [FILE] ...\n", argv [0]);
    return 1;
  }

  for (i = 1; i < argc; i++)
  {
    if (!strcmp (argv [i], "-v"))
      opt_verbose = TRUE;
    else
      test_extract_file (argv [i]);
  }

  g_blow_chunks ();

  return 0;
}
