/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* test-hash-collider.c - Test hashcash collisions
 *
 * Copyright (C) 2005 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include <glib.h>
#include <tnt.h>
#include <string.h>
#include <stdlib.h>

static gboolean opt_verbose = FALSE;

GMainLoop *loop = NULL;

static void
test_collider (gfloat work_rate)
{
  TntHashCollider *collider;
  TntStorage      *storage;

  g_print ("Running worker with work_rate = %.1f\n", work_rate);

  storage = tnt_storage_new (".", FALSE);
  collider = tnt_hash_collider_new (storage);

  tnt_hash_collider_set_target (collider, 42);
  tnt_hash_collider_set_work_rate (collider, work_rate);

  g_main_run (loop);
}

gint
main (gint argc, gchar *argv [])
{
  gint i;

  tnt_register_classes ();
  loop = g_main_new (FALSE);

  if (argc < 2)
  {
    g_print ("Usage: %s [-v] <utilization>\n"
             "       Utilization is a number in the range [0.0 .. 1.0].\n", argv [0]);
    return 1;
  }

  for (i = 1; i < argc; i++)
  {
    if (!strcmp (argv [i], "-v"))
      opt_verbose = TRUE;
    else
      test_collider (strtod (argv [i], NULL));
  }

  return 0;
}
